#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# PyMDstat
# Unitary test
#
# Copyright (C) 2014 Nicolargo <nicolas@nicolargo.com>

import unittest

from pymdstat import MdStat

# In Python 3, assertItemsEqual method is named assertCountEqual
try:
    unittest.TestCase.assertCountEqual = unittest.TestCase.assertItemsEqual
except AttributeError:
    pass


class TestPyMdStat(unittest.TestCase):

    """Test PyMDstat module."""

    def test_000_loadall(self):
        for i in range(1, 10):
            mdstat_test = MdStat('./tests/mdstat.0%s' % i)
            # print('%s' % mdstat_test.get_stats())
            self.assertNotEqual(mdstat_test.get_stats(), {})

    def test_099_didnotexist(self):
        try:
            mdstat_test = MdStat('/proc/NOmdstat')
        except IOError:
            self.assertTrue(True)
        else:
            self.assertFalse(True)

    def test_099_model(self):
        i = 4
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.personalities(), ['raid1', 'raid6', 'raid5', 'raid4'])
        self.assertCountEqual(mdstat_test.arrays(), ['md2', 'md3', 'md0', 'md1'])
        self.assertEqual(mdstat_test.type('md3'), 'raid5')
        self.assertEqual(mdstat_test.status('md3'), 'active')
        self.assertEqual(mdstat_test.available('md3'), 10)
        self.assertEqual(mdstat_test.used('md3'), 10)
        self.assertCountEqual(mdstat_test.components('md3'), ['sdk1', 'sdj1', 'sde1', 'sdl1', 'sdg1', 'sdf1', 'sdh1', 'sdc1', 'sdd1', 'sdi1'])
        self.assertEqual(mdstat_test.config('md3'), 'UUUUUUUUUU')

    def test_001(self):
        i = 1
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertEqual(mdstat_test.get_stats()['personalities'], [])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {})

    def test_002(self):
        i = 2
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid1', 'raid6', 'raid5', 'raid4'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md_d0': {'status': 'active', 'available': '5', 'used': '5', 'components': {'sdc1': '1', 'sdb1': '5', 'sde1': '0', 'sdd1': '2', 'sdf1': '4'}, 'config': 'UUUUU', 'type': 'raid5'}})

    def test_003(self):
        i = 3
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid6', 'raid5', 'raid4'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md0': {'status': 'active', 'available': '4', 'used': '3', 'components': {'sdb1': '1', 'sdd1': '2', 'sda1': '0'}, 'config': 'UUU_', 'type': 'raid5'}})

    def test_004(self):
        i = 4
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid1', 'raid6', 'raid5', 'raid4'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md2': {'status': 'active', 'available': '2', 'used': '2', 'components': {'sdb3': '1', 'sda3': '0'}, 'config': 'UU', 'type': 'raid1'}, 'md3': {'status': 'active', 'available': '10', 'used': '10', 'components': {'sdk1': '8', 'sdj1': '7', 'sde1': '2', 'sdl1': '9', 'sdg1': '4', 'sdf1': '3', 'sdh1': '5', 'sdc1': '0', 'sdd1': '1', 'sdi1': '6'}, 'config': 'UUUUUUUUUU', 'type': 'raid5'}, 'md0': {'status': 'active', 'available': '2', 'used': '2', 'components': {'sdb1': '1', 'sda1': '0'}, 'config': 'UU', 'type': 'raid1'}, 'md1': {'status': 'active', 'available': '2', 'used': '2', 'components': {'sdb2': '1', 'sda2': '0'}, 'config': 'UU', 'type': 'raid1'}})

    def test_005(self):
        i = 5
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid1', 'raid6', 'raid5', 'raid4'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md127': {'status': 'active', 'available': '6', 'used': '5', 'components': {'sde1': '2', 'sdg1': '4', 'sdf1': '3', 'sdh1': '6', 'sdc1': '0', 'sdd1': '1'}, 'config': 'UUUUU_', 'type': 'raid5'}})

    def test_006(self):
        i = 6
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['linear', 'raid0', 'raid1', 'raid5', 'raid4', 'raid6'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md0': {'status': 'active', 'available': '7', 'used': '7', 'components': {'sde1': '1', 'sdf1': '0', 'sdc1': '3', 'sdb1': '4', 'hdb1': '6', 'sdd1': '2', 'sda1': '5'}, 'config': 'UUUUUUU', 'type': 'raid6'}})

    def test_007(self):
        i = 7
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid1'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md1': {'status': 'active', 'available': '6', 'used': '4', 'components': {'sdc1': '2', 'sdb1': '4', 'sde1': '6', 'sdd1': '3', 'sdg1': '1'}, 'config': '_UUUU_', 'type': 'raid1'}})

    def test_008(self):
        i = 8
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['raid5'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md0': {'status': 'inactive', 'available': '4', 'used': '4', 'components': {'sdd1': '3', 'sdc1': '2', 'sda1': '0'}, 'config': 'UUUU', 'type': 'raid5'}})

    def test_009(self):
        i = 9
        mdstat_test = MdStat('./tests/mdstat.0%s' % i)
        self.assertCountEqual(mdstat_test.get_stats()['personalities'], ['linear', 'multipath', 'raid0', 'raid1', 'raid6', 'raid5', 'raid4', 'raid10'])
        self.assertEqual(mdstat_test.get_stats()['arrays'], {'md2': {'status': 'inactive', 'available': None, 'used': None, 'components': {'sdb': '0'}, 'config': None, 'type': None}, 'md0': {'status': 'active', 'available': '2', 'used': '2', 'components': {'sde1': '0', 'sdf1': '1'}, 'config': 'UU', 'type': 'raid1'}, 'md1': {'status': 'active', 'available': '2', 'used': '2', 'components': {'sde2': '0', 'sdf2': '1'}, 'config': 'UU', 'type': 'raid1'}})

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# PyMDstat
# ...
#
# Copyright (C) 2014 Nicolargo <nicolas@nicolargo.com>

__appname__ = "PyMDstat"
__version__ = "0.4.2"
__author__ = "Nicolas Hennion <nicolas@nicolargo.com>"
__licence__ = "MIT"

__all__ = ['MdStat']

from .pymdstat import MdStat



#!/usr/bin/env python

from setuptools import setup

data_files = [('share/doc/pymdstat', ['AUTHORS', 'LICENSE', 'NEWS', 'README.rst'])]

setup(
    name='pymdstat',
    version='0.4.2',
    description="Python library to parse Linux /proc/mdstat",
    long_description=open('README.rst').read(),
    author='Nicolas Hennion',
    author_email='nicolas@nicolargo.com',
    url='https://github.com/nicolargo/pymdstat',
    # download_url='https://s3.amazonaws.com/pymdstat/pymdstat-0.4.2.tar.gz',
    license="MIT",
    keywords="raid linux",
    packages=['pymdstat'],
    include_package_data=True,
    data_files=data_files,
    # test_suite="pymdstat.test",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Nicolargo <nicolas@nicolargo.com>
# License: MIT, see LICENSE for more details.

import sys
from functools import reduce
from re import split


class MdStat(object):

    """Main mdstat class."""

    def __init__(self, path='/proc/mdstat'):
        self.path = path
        self.content = ''

        # Stats will be stored in a dict
        self.stats = self.load()

    def __str__(self):
        """Return the content of the file."""
        return self.content

    def __repr__(self):
        """Return the content of the file."""
        return self.content

    def get_path(self):
        """Return the mdstat file path."""
        return self.path

    def get_stats(self):
        """Return the stats."""
        return self.stats

    def personalities(self):
        """Return the personalities (list)."""
        return self.get_stats()['personalities']

    def arrays(self):
        """Return the arrays (list)."""
        return self.get_stats()['arrays'].keys()

    def type(self, array):
        """Return the array's type."""
        return self.get_stats()['arrays'][array]['type']

    def status(self, array):
        """Return the array's status."""
        return self.get_stats()['arrays'][array]['status']

    def components(self, array):
        """Return the components of the arrays (list)."""
        return self.get_stats()['arrays'][array]['components'].keys()

    def available(self, array):
        """Return the array's available components number."""
        return int(self.get_stats()['arrays'][array]['available'])

    def used(self, array):
        """Return the array's used components number."""
        return int(self.get_stats()['arrays'][array]['used'])

    def config(self, array):
        """Return the array's config/status.

        U mean OK
        _ mean Failed
        """
        return self.get_stats()['arrays'][array]['config']

    def load(self):
        """Return a dict of stats."""
        ret = {}

        # Read the mdstat file
        with open(self.get_path(), 'r') as f:
            # lines is a list of line (with \n)
            lines = f.readlines()

        # First line: get the personalities
        # The "Personalities" line tells you what RAID level the kernel currently supports.
        # This can be changed by either changing the raid modules or recompiling the kernel.
        # Possible personalities include: [raid0] [raid1] [raid4] [raid5] [raid6] [linear] [multipath] [faulty]
        ret['personalities'] = self.get_personalities(lines[0])

        # Second to last before line: Array definition
        ret['arrays'] = self.get_arrays(lines[1:-1], ret['personalities'])

        # Save the file content as it for the __str__ method
        self.content = reduce(lambda x, y: x + y, lines)

        return ret

    def get_personalities(self, line):
        """Return a list of personalities readed from the input line."""
        return [split('\W+', i)[1] for i in line.split(':')[1].split(' ') if i.startswith('[')]

    def get_arrays(self, lines, personalities=[]):
        """Return a dict of arrays."""
        ret = {}

        i = 0
        while i < len(lines):
            try:
                # First array line: get the md device
                md_device = self.get_md_device_name(lines[i])
            except IndexError:
                # No array detected
                pass
            else:
                # Array detected
                if md_device is not None:
                    # md device line
                    ret[md_device] = self.get_md_device(lines[i], personalities)
                    # md config/status line
                    i += 1
                    ret[md_device].update(self.get_md_status(lines[i]))
            i += 1

        return ret

    def get_md_device(self, line, personalities=[]):
        """Return a dict of md device define in the line."""
        ret = {}

        splitted = split('\W+', line)
        # Raid status
        # Active or 'started'. An inactive array is usually faulty.
        # Stopped arrays aren't visible here.
        ret['status'] = splitted[1]
        if splitted[2] in personalities:
            # Raid type (ex: RAID5)
            ret['type'] = splitted[2]
            # Array's components
            ret['components'] = self.get_components(line, with_type=True)
        else:
            # Raid type (ex: RAID5)
            ret['type'] = None
            # Array's components
            ret['components'] = self.get_components(line, with_type=False)

        return ret

    def get_md_status(self, line):
        """Return a dict of md status define in the line."""
        ret = {}

        splitted = split('\W+', line)
        if len(splitted) < 7:
            ret['available'] = None
            ret['used'] = None
            ret['config'] = None
        else:
            # The final 2 entries on this line: [n/m] [UUUU_]
            # [n/m] means that ideally the array would have n devices however, currently, m devices are in use.
            # Obviously when m >= n then things are good.
            ret['available'] = splitted[-4]
            ret['used'] = splitted[-3]
            # [UUUU_] represents the status of each device, either U for up or _ for down.
            ret['config'] = splitted[-2]

        return ret

    def get_components(self, line, with_type=True):
        """Return a dict of components in the line.

        key: device name (ex: 'sdc1')
        value: device role number
        """
        ret = {}

        # Ignore (F) (see test 08)
        line2 = reduce(lambda x, y: x + y, split('\(.+\)', line))
        if with_type:
            splitted = split('\W+', line2)[3:]
        else:
            splitted = split('\W+', line2)[2:]
        ret = dict(zip(splitted[0::2], splitted[1::2]))

        return ret

    def get_md_device_name(self, line):
        """Return the md device name from the input line."""
        ret = split('\W+', line)[0]
        if ret.startswith('md'):
            return ret
        else:
            return None
